Rem COMMPORT.BAS
Rem C.D.Gregory, 6/30/93
Rem Together with COMSETUP.FRM, these modules take care of communications
Rem   chores for VB applications using a single COM port

Rem Revised 3/10/95 (comments)
Rem Revised 6/10/96 (handle multiple ports)
Rem                 NOTE: I have assumed that the port handle (nCid)
Rem                         is always the portnumber - 1 (e.g. Com3-->nCid=2)
Rem                         this may not always be true...

Option Explicit
Option Compare Text

Global PortToSetup As String
Global BaudRateToSetup As Single

Const ReadAmount = 512  ' max input buffer size

Rem The following global variables take care of information
Rem       from other VB modules.
Rem NONE

Rem Local variables for this module
Type DCB
    Id As String * 1
    BaudRate As Integer
    ByteSize As String * 1
    Parity As String * 1
    StopBits As String * 1
    RlsTimeout As Integer
    CtsTimeout As Integer
    DsrTimeout As Integer
    Bits1 As String * 1 ' The fifteen actual DCB bit-sized data fields
    Bits2 As String * 1 ' within these two bytes can be manipulated by
                          ' bitwise logical And/Or operations.  Refer to
                          ' SDKWIN.HLP for location/meaning of specific bits
    XonChar As String * 1
    XoffChar As String * 1
    XonLim As Integer
    XoffLim As Integer
    PeChar As String * 1
    EofChar As String * 1
    EvtChar As String * 1
    TxDelay As Integer
End Type

Type ComStat
    Bits As String * 1 ' For specific bit flags and their
                         ' meanings, refer to SDKWIN.HLP.
    cbInQue As Integer
    cbOutQue As Integer
End Type


Dim ComErrCode As Integer, nChar As Integer
Dim nCerr As Integer, nCread As Integer
Dim BytesToSend As Integer, wInQueue  As Integer, wOutQueue As Integer
Dim ComErr As ComStat
Dim ComDCB(4)  As DCB
Dim RS232InitFlag(4) As Integer


'  COMM declarations
'
Declare Function OpenComm Lib "User" (ByVal lpComName As String, ByVal wInQueue As Integer, ByVal wOutQueue As Integer) As Integer
Declare Function SetCommState Lib "User" (anyDCB As DCB) As Integer
Declare Function GetCommState Lib "User" (ByVal nCid As Integer, lpDCB As DCB) As Integer
Declare Function GetCommError Lib "User" (ByVal nCid As Integer, lpCs As ComStat) As Integer
Declare Function ReadComm Lib "User" (ByVal nCid As Integer, ByVal lpBuf As String, ByVal nSize As Integer) As Integer
Declare Function UngetCommChar Lib "User" (ByVal nCid As Integer, ByVal cChar As Integer) As Integer
Declare Function WriteComm Lib "User" (ByVal nCid As Integer, ByVal lpBuf As String, ByVal nSize As Integer) As Integer
Declare Function CloseComm Lib "User" (ByVal nCid As Integer) As Integer
Declare Function BuildCommDCB Lib "User" (ByVal lpDef As String, lpDCB As DCB) As Integer
Declare Function TransmitCommChar Lib "User" (ByVal nCid As Integer, ByVal cChar As Integer) As Integer
Declare Function SetCommEventMask Lib "User" (ByVal nCid As Integer, nEvtMask As Integer) As Long
Declare Function GetCommEventMask Lib "User" (ByVal nCid As Integer, ByVal nEvtMask As Integer) As Integer
Declare Function SetCommBreak Lib "User" (ByVal nCid As Integer) As Integer
Declare Function ClearCommBreak Lib "User" (ByVal nCid As Integer) As Integer
Declare Function FlushComm Lib "User" (ByVal nCid As Integer, ByVal nQueue As Integer) As Integer
Declare Function EscapeCommFunction Lib "User" (ByVal nCid As Integer, ByVal nFunc As Integer) As Integer

Const NOPARITY = 0
Const ODDPARITY = 1
Const EVENPARITY = 2
Const MARKPARITY = 3
Const SPACEPARITY = 4

Const ONESTOPBIT = 0
Const ONE5STOPBITS = 1
Const TWOSTOPBITS = 2

Const IGNORE = 0 '  Ignore signal
Const INFINITE = &HFFFF  '  Infinite timeout

'  Error Flags
Const CE_RXOVER = &H1    '  Receive Queue overflow
Const CE_OVERRUN = &H2   '  Receive Overrun Error
Const CE_RXPARITY = &H4  '  Receive Parity Error
Const CE_FRAME = &H8 '  Receive Framing error
Const CE_BREAK = &H10    '  Break Detected
Const CE_CTSTO = &H20    '  CTS Timeout
Const CE_DSRTO = &H40    '  DSR Timeout
Const CE_RLSDTO = &H80   '  RLSD Timeout
Const CE_TXFULL = &H100  '  TX Queue is full
Const CE_PTO = &H200 '  LPTx Timeout
Const CE_IOE = &H400 '  LPTx I/O Error
Const CE_DNS = &H800 '  LPTx Device not selected
Const CE_OOP = &H1000    '  LPTx Out-Of-Paper
Const CE_MODE = &H8000   '  Requested mode unsupported

Const IE_BADID = (-1)    '  Invalid or unsupported id
Const IE_OPEN = (-2) '  Device Already Open
Const IE_NOPEN = (-3)    '  Device Not Open
Const IE_MEMORY = (-4)   '  Unable to allocate queues
Const IE_DEFAULT = (-5)  '  Error in default parameters
Const IE_HARDWARE = (-10)    '  Hardware Not Present
Const IE_BYTESIZE = (-11)    '  Illegal Byte Size
Const IE_BAUDRATE = (-12)    '  Unsupported BaudRate

'  Events
Const EV_RXCHAR = &H1    '  Any Character received
Const EV_RXFLAG = &H2    '  Received certain character
Const EV_TXEMPTY = &H4   '  Transmitt Queue Empty
Const EV_CTS = &H8   '  CTS changed state
Const EV_DSR = &H10  '  DSR changed state
Const EV_RLSD = &H20 '  RLSD changed state
Const EV_BREAK = &H40    '  BREAK received
Const EV_ERR = &H80  '  Line status error occurred
Const EV_RING = &H100    '  Ring signal detected
Const EV_PERR = &H200    '  Printer error occured

'  Escape Functions
Const SETXOFF = 1    '  Simulate XOFF received
Const SETXON = 2 '  Simulate XON received
Const SETRTS = 3 '  Set RTS high
Const CLRRTS = 4 '  Set RTS low
Const SETDTR = 5 '  Set DTR high
Const CLRDTR = 6 '  Set DTR low
Const RESETDEV = 7   '  Reset device if possible

Const LPTx = &H80    '  Set if ID is for LPT device

Function ClearCommPort (PortID As Integer) As Integer
Rem Clear the buffer without returning the values

Dim CharBuffer As String

Rem Check the comm port for incoming data
    CharBuffer = Space$(ReadAmount + 1)  ' leave room for a terminal null ?
Rem Always check the error status at the same time to keep the port open
    nChar = ReadComm(PortID, CharBuffer, ReadAmount)
    ComErrCode = GetCommError(PortID, ComErr)
Rem Error handling
    Select Case nChar ' <=0 means error
        Case Is = 0 ' Queue probably empty
            If ComErrCode <> 0 Then
                ShowCommErrors
            End If
        Case Is < 0 ' Got some characters with an error
            If ComErrCode <> 0 Then
                ShowCommErrors
            End If
        Case Is > 0 ' Got some legal characters
    End Select
    
    nChar = Abs(nChar)

ClearCommPort = nChar

End Function

Sub CloseCommPort (PortID As Integer)
Rem Don't forget to call this when changing ports or exiting !
Dim OK As Integer
    
    RS232InitFlag(PortID) = RS232InitFlag(PortID) - 1
    If RS232InitFlag(PortID) = 0 Then
        OK = CloseComm(PortID)
    End If

End Sub

Function GetCommString (PortID As Integer, Characters As String) As Integer
Rem Function returns the number of characters in the string (could be 0)

Dim ReadAmount As Integer, CharBuffer As String

Rem Check the comm port for incoming data
    ReadAmount = 512
    CharBuffer = Space$(ReadAmount + 1)  ' leave room for a terminal null ?
Rem Always check the error status at the same time to keep the port open
    nChar = ReadComm(PortID, CharBuffer, ReadAmount)
    ComErrCode = GetCommError(PortID, ComErr)
Rem Error handling
    Select Case nChar ' <=0 means error
        Case Is = 0 ' Queue probably empty
            If ComErrCode <> 0 Then
                ShowCommErrors
            End If
        Case Is < 0 ' Got some characters with an error
            If ComErrCode <> 0 Then
                ShowCommErrors
            End If
        Case Is > 0 ' Got some legal characters
    End Select
    
    nChar = Abs(nChar)
    If nChar > 0 Then ' Return characters anyway, even if error!
        Characters = Left$(CharBuffer, nChar)
    End If

GetCommString = nChar

End Function

Function InitCommPort (PortName As String, BaudRate As Single) As Integer
Rem This routine should be called at the start of any application using a COM port

Rem The calling routine sets up :
Rem     Port (PortName, e.g. "COM1")
Rem     Baud (BaudRate, e.g. 4800)
Rem Returns the CommPortID from Windows

Dim PortIndex As Integer
Dim WhatToDo As Integer

PortIndex = Val(Mid$(PortName, 4, 1)) - 1

If RS232InitFlag(PortIndex) Then   'keep track of number of users
    RS232InitFlag(PortIndex) = RS232InitFlag(PortIndex) + 1
    InitCommPort = PortIndex ' fake it!
    Exit Function
End If

Rem Here we specify how much space will be reserved for the input and output queues
    wInQueue = 2048
    wOutQueue = 1024

Rem Try to open the port
Dim nCid As Integer

reopen:
    nCid = OpenComm(PortName, wInQueue, wOutQueue)
    Select Case nCid
        Case IE_OPEN ' Already open
            WhatToDo = MsgBox("Port " & PortName & " already open - Continue ?", MB_OKCANCEL + MB_ICONEXCLAMATION, "InitPortName")
            If WhatToDo = 1 Then ' This is a bit risky, but....take a guess at the ID and....
                nCerr = CloseComm(PortIndex)
                If nCerr < 0 Then
                    WhatToDo = MsgBox("Could not close port - giving up", MB_OK, "InitPortName")
                    Exit Function
                End If
                GoTo reopen
            Else
                Exit Function
            End If
        Case IE_BADID
            WhatToDo = MsgBox("Bad device name :" & PortName, MB_OK, "InitPortName")
            Exit Function
        Case IE_BAUDRATE
            WhatToDo = MsgBox("Bad baud rate :" & Str$(BaudRate), MB_OK, "InitPortName")
            Exit Function
        Case IE_BYTESIZE
            WhatToDo = MsgBox("Bad byte size :" & PortName, MB_OK, "InitPortName")
            Exit Function
        Case IE_HARDWARE
            WhatToDo = MsgBox("No hardware for :" & PortName, MB_OK, "InitPortName")
            Exit Function
        Case IE_MEMORY
            WhatToDo = MsgBox("Not enough memory :" & Str$(wInQueue + wOutQueue) & " bytes requested", MB_OK, "InitPortName")
            Exit Function
        Case IE_NOPEN
            WhatToDo = MsgBox("Could not open :" & PortName, MB_OK, "InitPortName")
            Exit Function
        Case IE_BADID
            WhatToDo = MsgBox("Bad device name :" & PortName, MB_OK, "InitPortName")
            Exit Function
        Case Is < 0 ' Other error
                MsgBox "Comm Port Open Error =" & Str$(nCid), MB_OK, "InitPortName"
                End
        Case Else ' OK ! nCid is a positive number
    End Select

Rem Now initialize the port properly - fill in the DCB
    ComDCB(PortIndex).Id = Chr$(nCid)
    ComDCB(PortIndex).BaudRate = BaudRate
    ComDCB(PortIndex).ByteSize = Chr$(8)
    ComDCB(PortIndex).Parity = Chr$(NOPARITY)
    ComDCB(PortIndex).StopBits = Chr$(TWOSTOPBITS)
    ComDCB(PortIndex).RlsTimeout = 0       'msec
    ComDCB(PortIndex).CtsTimeout = 0       'msec
    ComDCB(PortIndex).DsrTimeout = 0       'msec

    ComDCB(PortIndex).Bits1 = Chr$(&H7)
    ComDCB(PortIndex).Bits2 = Chr$(&H0)    ' parity errors always show up after 1000 characters - ignore them

    ComDCB(PortIndex).XonChar = Chr$(&H11)
    ComDCB(PortIndex).XoffChar = Chr$(&H13)
    ComDCB(PortIndex).XonLim = 10
    ComDCB(PortIndex).XoffLim = 1000
    ComDCB(PortIndex).PeChar = "?"
    ComDCB(PortIndex).EofChar = Chr$(&H1A)
    ComDCB(PortIndex).EvtChar = Chr$(&H0)
    ComDCB(PortIndex).TxDelay = 0

    nCerr = SetCommState(ComDCB(PortIndex))
    If nCerr < 0 Then
        MsgBox "SetCommState (DCB) Error for " & PortName & " : Error " & Str$(nCerr), 1, "InitPortName"
        Exit Function
    End If

Rem Here, if successful
RS232InitFlag(PortIndex) = 1 ' first user only
InitCommPort = nCid

End Function

Sub PutCommString (PortID As Integer, CommString As String)
Dim OK As Integer, I As Integer, C As String
Rem Convert any "^" to control chars 6/25/96 CDG for WPRairie

If CommString = "" Then Exit Sub

If InStr(CommString, "^") > 0 Then
    For I = 1 To Len(CommString)
        C = Mid$(CommString, I, 1)
        If C = "^" Then
            I = I + 1
            C = Chr$(Asc(Mid$(CommString, I, 1)) - 64)
        End If
        OK = WriteComm(PortID, C, 1)
        ComErrCode = GetCommError(PortID, ComErr)
        If OK < 0 Then
            ShowCommErrors
        End If
    Next I
Else
    OK = WriteComm(PortID, CommString, Len(CommString))
    ComErrCode = GetCommError(PortID, ComErr)
    If OK < 0 Then
        ShowCommErrors
    End If
End If

End Sub

Sub ShowCommErrors ()
Rem Display Comm port error status (and fix if necessary)
Dim ErrorMsg As String, CRLF As String

CRLF = Chr$(10) & Chr$(13)

Select Case ComErrCode
    Case CE_BREAK
        ErrorMsg = "Break Detected"
    Case CE_CTSTO
        ErrorMsg = "CTS timeout"
    Case CE_DNS
        ErrorMsg = "Parallel device not selected"
    Case CE_DSRTO
        ErrorMsg = "DSR timeout"
    Case CE_FRAME
        ErrorMsg = "Framing error"
    Case CE_IOE
        ErrorMsg = "Parallel I/O error"
    Case CE_MODE
        ErrorMsg = "Invalid mode or device ID"
    Case CE_OOP
        ErrorMsg = "Parallel device out of paper"
    Case CE_OVERRUN
        ErrorMsg = "Character overrun"
    Case CE_PTO
        ErrorMsg = "Parallel timeout"
    Case CE_RLSDTO
        ErrorMsg = "RLSD timeout"
    Case CE_RXOVER
        ErrorMsg = "Receiver Queue overflow"
        'FlushRx = 1
        'ApiFlush = FlushComm(nCid, FlushRx)
    Case CE_RXPARITY
        ErrorMsg = "Parity error"
    Case CE_TXFULL
        ErrorMsg = "Transmit Queue full"
    Case Else
        ErrorMsg = "Unknown GetCommError Status"
End Select

ErrorMsg = ErrorMsg & CRLF
ErrorMsg = ErrorMsg & CRLF & "ReadComm Status : " & Str$(nChar) & CRLF & "GetCommError Status : 0x" & Hex$(ComErrCode) & CRLF
ErrorMsg = ErrorMsg & "Flags : 0x" & Hex$(Asc(ComErr.Bits)) & CRLF & "wOutQue : " & Str$(ComErr.cbOutQue) & "   wInQue : " & Str$(ComErr.cbInQue)

MsgBox ErrorMsg, MB_OK, "ShowCommErrors"

End Sub

Function WaitForTxReady (PortID As Integer) As Integer

    ComErrCode = GetCommError(PortID, ComErr)
    If ComErr.cbOutQue > 0 Then
        WaitForTxReady = False
    Else
        WaitForTxReady = True
    End If
    DoEvents

End Function

