Rem "Dummy rotor" controller calibration and test
Rem Replace the module "PRRIEDRV.BAS" with this

Rem  CDGregory, K8CG, 6/21/95
Rem  Rev 6/21/95 - add constants for conversion of ADC to Az and El
Rem  Rev 7/11/95 - for WPRAIRIE
Rem  Rev 7/7/96 - update subroutine calls
Rem  Rev 7/9/96 - make ADC readings work for any calibration values

Option Explicit
Option Compare Text

Global BaseAddress ' retained for compatibility only

Type ADC
    Channel As Integer
    Value As Integer
    PrevValue As Integer
    
    Gain As Single      ' conversion to volts
    Offset As Single
    Volts As Single
End Type

Rem Dummy driver local variables
Dim AzStartTime  As Double, AzRunning As Integer
Dim ElStartTime  As Double, ElRunning As Integer
Dim AzADC As Integer
Dim ElADC As Integer

Dim AzDirectionBit As Integer
Dim ElDirectionBit As Integer

Rem Control bits for rotators
Global RotateUp As Integer
Global RotateDn As Integer
Global RotateCW As Integer
Global RotateCCW As Integer

Sub AssignADCCal (AD As ADC)

Rem Initialize the ADC calibrations
AD.Gain = GetFloatingPref("PrairieBoard", "ADC" & Format$(AD.Channel) & "Gain", "0.0196")
AD.Offset = GetFloatingPref("PrairieBoard", "ADC" & Format$(AD.Channel) & "Offset", "0")

End Sub

Sub InitializePrairieParams ()
Rem Get preferences from .INI file
Rem Called at startup only

BaseAddress = GetIntegerPref("PrairieBoard", "BaseAddress", "772")

Az.ADC.Channel = GetIntegerPref("PrairieBoard", "AzChannel", "0")
El.ADC.Channel = GetIntegerPref("PrairieBoard", "ElChannel", "1")
AssignADCCal Az.ADC
AssignADCCal El.ADC

End Sub

Sub InitPrairieHdw ()
Rem Dummy call

AzADC = 174
ElADC = 40

End Sub

Sub PrairieTune (Updown As String)
Static FreqVal As Integer, DirectionBit As Integer

If Updown = "Up" Then
    DirectionBit = 1
Else
    DirectionBit = -1
End If

FreqVal = DirectionBit + FreqVal
Debug.Print FreqVal

End Sub

Function ReadADC (Index As Integer) As Integer
Rem This routine actually simulates the "rotation"

Rem Read the ADC and return a number (0-255)
Dim Value As Single
Dim SampleTime As Double, TimeDiff As Double
Dim Position As Single

SampleTime = Now

If Index = Az.ADC.Channel Then ' azimuth
    If AzRunning Then
        TimeDiff = SampleTime - AzStartTime
        Position = ADCToAngle(Az)
        Position = Position + 86400 * TimeDiff * AzDirectionBit
        If Position > Az.CWLimit Then Position = Az.CWLimit
        If Position < Az.CCWLimit Then Position = Az.CCWLimit
        Rem Convert to volts
        Value = (Position - Az.CCWLimit) * (Az.CWVolts - Az.CCWVolts) / (Az.CWLimit - Az.CCWLimit) + Az.CCWVolts
        Rem convert to ADC
        Value = (Value - Az.ADC.Offset) / Az.ADC.Gain
    Else
        Value = AzADC
    End If
    AzADC = Value
    ReadADC = Value
Else              ' elevation
    If ElRunning Then
        TimeDiff = SampleTime - ElStartTime
        Position = ADCToAngle(El)
        Position = Position + 86400 * TimeDiff * ElDirectionBit
        If Position > El.CWLimit Then Position = El.CWLimit
        If Position < El.CCWLimit Then Position = El.CCWLimit
        Rem Convert to volts
        Value = (Position - El.CCWLimit) * (El.CWVolts - El.CCWVolts) / (El.CWLimit - El.CCWLimit) + El.CCWVolts
        Rem convert to ADC
        Value = (Value - El.ADC.Offset) / El.ADC.Gain
    Else
        Value = ElADC
    End If
    ElADC = Value
    ReadADC = Value
End If

End Function

Sub RotateAzimuth (LeftRight As String, Mode As String)

If LeftRight = "right" Then
    AzDirectionBit = 1
Else
    AzDirectionBit = -1
End If

If Mode = "Start" Then
    AzStartTime = Now
    AzRunning = True
Else 'end
    AzRunning = False
End If

End Sub

Sub RotateElevation (Updown As String, Mode As String)

If Updown = "Up" Then
    ElDirectionBit = 1
Else
    ElDirectionBit = -1
End If

If Mode = "Start" Then
    ElStartTime = Now
    ElRunning = True
Else 'end
    ElRunning = False
End If

End Sub

