Rem Rotors.Bas  7/15/95 K8CG
Rem This is the main antenna tracking routine
Rem 7/5/96 - major revision to data structure and var names


Option Explicit
Option Compare Text

Type Rotator
    ADC As ADC
    CWLimit As Single
    CCWLimit As Single
    CWVolts As Single
    CCWVolts As Single
    Target As Single
    Actual As Single
    Deadband As Single
    Enabled As Integer
    Moving As Integer
    StallCount As Integer
    LastPos As Single
    CurrentPos As Single
    ManualOverRide As Integer
End Type

Global Az As Rotator, El As Rotator

Global AntennaControlType As String
Global AntennaCommPort As String
Global AntennaBaudRate As Single
Global AntennaCommPortID As Integer

Global MinElevation As Single
Global StartElevation As Single

Rem Flags
'Global Rotor_El180_OK  As Integer
Global Const StallThresh = 20   ' 6 to 10 seconds

Function ADCToAngle (AzEl As Rotator)
Rem Convert the ADC reading to an angle using the cal values

AzEl.ADC.Volts = ADCToVolts(AzEl.ADC)
ADCToAngle = AzEl.CCWLimit + (AzEl.CWLimit - AzEl.CCWLimit) * (AzEl.ADC.Volts - AzEl.CCWVolts) / (AzEl.CWVolts - AzEl.CCWVolts)

End Function

Function ADCToVolts (AD As ADC) As Single
Rem Convert the ADC reading to a voltage

ADCToVolts = AD.Offset + AD.Gain * AD.Value

End Function

Function CheckStall (AzEl As Rotator) As Integer
Rem Check for stall (no change in position)

If AzEl.CurrentPos = AzEl.LastPos Then ' maybe stall
    AzEl.StallCount = AzEl.StallCount + 1
    If AzEl.StallCount > StallThresh Then
        CheckStall = True
    Else
        CheckStall = False
    End If
Else ' no stall
    AzEl.StallCount = 0
    CheckStall = False
End If

End Function

Sub InitializeRotorParams ()
Rem Get preferences from .INI file
Rem Called at startup only

AntennaControlType = GetStringPref("Rotors", "ControlType", "None")
AntennaCommPort = GetStringPref("Rotors", "CommPort", "COM1:")
AntennaBaudRate = GetIntegerPref("Rotors", "BaudRate", "9600")
El.CCWLimit = GetIntegerPref("Rotors", "ElCCWLimit", "0")
El.CWLimit = GetIntegerPref("Rotors", "ElCWLimit", "180")
frmControl!hscrManualControl(1).Min = El.CCWLimit
frmControl!hscrManualControl(1).Max = El.CWLimit

Az.CCWLimit = GetIntegerPref("Rotors", "AzCCWLimit", "-180")
Az.CWLimit = GetIntegerPref("Rotors", "AzCWLimit", "179")
frmControl!hscrManualControl(0).Min = Az.CCWLimit
frmControl!hscrManualControl(0).Max = Az.CWLimit

'Rotor_El180_OK = GetLogicalPref("Rotors", "El_180_OK", True)

Az.Enabled = GetLogicalPref("Rotors", "AzEnabled", False)
El.Enabled = GetLogicalPref("Rotors", "ElEnabled", False)

Rem Initialize the rotor calibrations
El.CCWVolts = GetFloatingPref("Rotors", "ElCCWVolts", "0")
El.CWVolts = GetFloatingPref("Rotors", "ElCWVolts", "180")
Az.CCWVolts = GetFloatingPref("Rotors", "AzCCWVolts", "-180")
Az.CWVolts = GetFloatingPref("Rotors", "AzCWVolts", "179")

Az.Deadband = GetFloatingPref("Rotors", "Deadband", "3")
El.Deadband = GetFloatingPref("Rotors", "Deadband", "3")
MinElevation = GetFloatingPref("Rotors", "MinElevation", "0")
StartElevation = GetFloatingPref("Rotors", "StartElevation", "-5")

End Sub

Sub InitRotorHdw ()
Rem Set up any other rotor hardware here at startup

If AntennaControlType = "Serial" Then
    AntennaCommPortID = InitCommPort(AntennaCommPort, AntennaBaudRate)
End If

End Sub

Function LimitSetpoint (Setpoint As Single, AzEl As Rotator) As Integer

Select Case Setpoint
    Case Is < AzEl.CCWLimit
        LimitSetpoint = AzEl.CCWLimit
    Case Is > AzEl.CWLimit
        LimitSetpoint = AzEl.CWLimit
    Case Else
        LimitSetpoint = Setpoint
End Select

End Function

Function PositionError (Actual As Single, Setpoint As Single) As Single
Rem Compute the difference between the set and actual positions
Rem    regardless of the Az range (-180 to 180 or 0 to 360)
Rem A positive difference implies counter-clockwise rotation required
Rem    (i.e. try to reduce the angle)

Dim Temp1 As Single, Temp2 As Single

Temp1 = PositiveAngle(Actual)
Temp2 = PositiveAngle(Setpoint)

If Az.CCWLimit < 0 Then    ' typically -180....
    If Temp1 > 180 Then Temp1 = Temp1 - 360
    If Temp2 > 180 Then Temp2 = Temp2 - 360
End If

PositionError = Temp1 - Temp2


End Function

Function PositiveAngle (Argument As Single) As Single

If Argument < 0 Then
    PositiveAngle = Argument + 360
Else
    PositiveAngle = Argument
End If

End Function

Sub StartAz ()
Rem Decide which way to move, and turn on the rotor

If PositionError(Az.CurrentPos, Val(frmControl!lblSet(0).Caption)) < 0 Then
    Rem rotate right (to larger angle number)
    RotateAzimuth "right", "Start"
    Debug.Print "rotate right"
Else ' rotate left
    RotateAzimuth "left", "Start"
    Debug.Print "rotate left"
End If
Az.Moving = True

End Sub

Sub StartEl ()
Rem Decide which way to move, and turn on the rotor

If El.CurrentPos - Val(frmControl!lblSet(1).Caption) < 0 Then
    Rem rotate up (to larger angle number)
    RotateElevation "up", "Start"
Else ' rotate down
    RotateElevation "down", "Start"
End If
El.Moving = True

End Sub

Sub StopAz ()
Rem Shut off Az rotor and set flag

RotateAzimuth "", "End"
Az.Moving = False

End Sub

Sub StopEl ()
Rem Shut off Az rotor and set flag

RotateElevation "", "End"
El.Moving = False

End Sub

Sub UpdatePhysicalAntennas ()
Rem Here we try to match antenna positions with the
Rem   target values

If Not Az.ManualOverRide Then
    If Az.Enabled Then
        If Az.Moving Then
            If CheckStall(Az) Then
                StopAz
                frmDiags!lblStatus.Caption = "Az Rotor Stalled"
                frmControl!chkEnabled(0).Value = 0
            End If
            If Abs(PositionError(Az.CurrentPos, Val(frmControl!lblSet(0).Caption))) <= Abs(Az.Deadband) Then StopAz
        Else ' not moving
            If Abs(PositionError(Az.CurrentPos, Val(frmControl!lblSet(0).Caption))) > Abs(Az.Deadband) Then StartAz
        End If
    Else ' AZ not enabled, make sure it's off
        StopAz
    End If
End If

If Not El.ManualOverRide Then
    If El.Enabled Then
        If El.Moving Then
            If CheckStall(El) Then
                StopEl
                frmDiags!lblStatus.Caption = "El Rotor Stalled"
                frmControl!chkEnabled(1).Value = 0
            End If
            If Abs(El.CurrentPos - Val(frmControl!lblSet(1).Caption)) <= Abs(El.Deadband) Then StopEl
        Else ' not moving
            If Abs(El.CurrentPos - Val(frmControl!lblSet(1).Caption)) > Abs(El.Deadband) Then StartEl
        End If
    Else ' El not enabled, make sure it's off
        StopEl
    End If
End If

End Sub

